@@ -1,26 +1,17 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-98 by Solar Designer
+ * Copyright (c) 1996-2001 by Solar Designer
  */
 
 #include <string.h>
 
 #include "arch.h"
+#include "common.h"
 #include "DES_std.h"
 
-#if ARCH_GENERIC
-
-#if ((__GNUC__ == 2) && (__GNUC_MINOR__ >= 7)) || (__GNUC__ > 2)
-#define DES_ALIGN			__attribute__ ((aligned (128)))
-#else
-#define DES_ALIGN			/* nothing */
-#endif
-
-#endif
-
 #if ARCH_BITS >= 64
 
-#if ARCH_GENERIC
+#if !DES_ASM
 
 static union {
 	double dummy;
@@ -31,7 +22,7 @@ static union {
 		ARCH_WORD SPE_W[4][64 * 64];
 #endif
 	} data;
-} DES_ALIGN DES_all;
+} CC_CACHE_ALIGN DES_all;
 
 #else
 
@@ -46,7 +37,7 @@ static ARCH_WORD DES_SPE[8][64];
 
 #else
 
-#if ARCH_GENERIC
+#if !DES_ASM
 
 static union {
 	double dummy;
@@ -60,7 +51,7 @@ static union {
 		ARCH_WORD SPE_W[4][64 * 64][2];
 #endif
 	} data;
-} DES_ALIGN DES_all;
+} CC_CACHE_ALIGN DES_all;
 
 #else
 
@@ -80,7 +71,7 @@ static ARCH_WORD DES_SPE[8][64][2];
 
 #endif
 
-#if ARCH_GENERIC
+#if !DES_ASM
 
 #define DES_KS_copy			DES_all.data.KS
 #define DES_SPE_F			DES_all.data.SPE_F
@@ -91,8 +82,8 @@ static ARCH_WORD DES_SPE[8][64][2];
 DES_binary DES_IV;
 ARCH_WORD DES_count;
 
-DES_KS DES_ALIGN DES_KS_current;
-DES_KS DES_ALIGN DES_KS_table[8][128];
+DES_KS CC_CACHE_ALIGN DES_KS_current;
+DES_KS CC_CACHE_ALIGN DES_KS_table[8][128];
 
 #endif
 
@@ -100,7 +91,7 @@ static int DES_KS_updates;
 static char DES_key[16];
 
 #if DES_COPY
-#if !ARCH_GENERIC
+#if DES_ASM
 extern DES_KS DES_KS_copy;
 #endif
 unsigned ARCH_WORD *DES_out;
@@ -122,7 +113,7 @@ static ARCH_WORD DES_IP_E[8][16][2], DES_C_FP[16][16][2];
 #if (ARCH_BITS >= 64 && (DES_SCALE || !DES_MASK) || DES_128K) && \
 	DES_SIZE_FIX == 2
 /*
- * 64 bit architectures which can shift addresses left by 1 bit with no extra
+ * 64-bit architectures which can shift addresses left by 1 bit with no extra
  * time required (for example by adding a register to itself).
  */
 #define DES_INDEX(SPE, i) \
@@ -130,7 +121,7 @@ static ARCH_WORD DES_IP_E[8][16][2], DES_C_FP[16][16][2];
 #else
 #if DES_SIZE_FIX == 0
 /*
- * 64 bit architectures which can shift addresses left by 3 bits (but maybe
+ * 64-bit architectures which can shift addresses left by 3 bits (but maybe
  * not by 1) with no extra time required (for example by using the S8ADDQ
  * instruction on DEC Alphas; we would need an ADDQ anyway).
  */
@@ -160,7 +151,7 @@ static ARCH_WORD DES_IP_E[8][16][2], DES_C_FP[16][16][2];
 #if DES_MASK
 #if ARCH_BITS >= 64 && !DES_SCALE && DES_SIZE_FIX == 2
 /*
- * This method might be good for some 64 bit architectures with no complicated
+ * This method might be good for some 64-bit architectures with no complicated
  * addressing modes supported. It would be the best one for DEC Alphas if they
  * didn't have the S8ADDQ instruction.
  */
@@ -192,12 +183,18 @@ static ARCH_WORD DES_IP_E[8][16][2], DES_C_FP[16][16][2];
 #endif
 #define DES_KS_INDEX(i)			(DES_KS_copy + (i * (16 / DES_SIZE)))
 
-#define DES_TO_6_BIT(x) \
+#define DES_24_TO_32(x) \
 	(((x) & 077) | \
 	(((x) & 07700) << 2) | \
 	(((x) & 0770000) << 4) | \
 	(((x) & 077000000) << 6))
 
+#define DES_16_TO_32(x) \
+	((((x) & 0xF) << 1) | \
+	(((x) & 0xF0) << 5) | \
+	(((x) & 0xF00) << 9) | \
+	(((x) & 0xF000) << 13))
+
 #if DES_128K
 #define DES_UNDO_SIZE_FIX(x) \
 	((((x) >> 1) & 0xFF00FF00) | (((x) & 0x00FF00FF) >> 3))
@@ -294,58 +291,33 @@ static unsigned char DES_C[64] = {
 	44, 45, 46, 47, 60, 61, 62, 63
 };
 
-unsigned char DES_K_bits[0x300] = {
-	9, 50, 33, 59, 48, 16, 32, 56, 1, 8, 18, 41, 2, 34, 25, 24,
-	43, 57, 58, 0, 35, 26, 17, 40, 21, 27, 38, 53, 36, 3, 46, 29,
-	4, 52, 22, 28, 60, 20, 37, 62, 14, 19, 44, 13, 12, 61, 54, 30,
-	1, 42, 25, 51, 40, 8, 24, 48, 58, 0, 10, 33, 59, 26, 17, 16,
-	35, 49, 50, 57, 56, 18, 9, 32, 13, 19, 30, 45, 28, 62, 38, 21,
-	27, 44, 14, 20, 52, 12, 29, 54, 6, 11, 36, 5, 4, 53, 46, 22,
-	50, 26, 9, 35, 24, 57, 8, 32, 42, 49, 59, 17, 43, 10, 1, 0,
-	48, 33, 34, 41, 40, 2, 58, 16, 60, 3, 14, 29, 12, 46, 22, 5,
-	11, 28, 61, 4, 36, 27, 13, 38, 53, 62, 20, 52, 19, 37, 30, 6,
-	34, 10, 58, 48, 8, 41, 57, 16, 26, 33, 43, 1, 56, 59, 50, 49,
-	32, 17, 18, 25, 24, 51, 42, 0, 44, 54, 61, 13, 27, 30, 6, 52,
-	62, 12, 45, 19, 20, 11, 60, 22, 37, 46, 4, 36, 3, 21, 14, 53,
-	18, 59, 42, 32, 57, 25, 41, 0, 10, 17, 56, 50, 40, 43, 34, 33,
-	16, 1, 2, 9, 8, 35, 26, 49, 28, 38, 45, 60, 11, 14, 53, 36,
-	46, 27, 29, 3, 4, 62, 44, 6, 21, 30, 19, 20, 54, 5, 61, 37,
-	2, 43, 26, 16, 41, 9, 25, 49, 59, 1, 40, 34, 24, 56, 18, 17,
-	0, 50, 51, 58, 57, 48, 10, 33, 12, 22, 29, 44, 62, 61, 37, 20,
-	30, 11, 13, 54, 19, 46, 28, 53, 5, 14, 3, 4, 38, 52, 45, 21,
-	51, 56, 10, 0, 25, 58, 9, 33, 43, 50, 24, 18, 8, 40, 2, 1,
-	49, 34, 35, 42, 41, 32, 59, 17, 27, 6, 13, 28, 46, 45, 21, 4,
-	14, 62, 60, 38, 3, 30, 12, 37, 52, 61, 54, 19, 22, 36, 29, 5,
-	35, 40, 59, 49, 9, 42, 58, 17, 56, 34, 8, 2, 57, 24, 51, 50,
-	33, 18, 48, 26, 25, 16, 43, 1, 11, 53, 60, 12, 30, 29, 5, 19,
-	61, 46, 44, 22, 54, 14, 27, 21, 36, 45, 38, 3, 6, 20, 13, 52,
-	56, 32, 51, 41, 1, 34, 50, 9, 48, 26, 0, 59, 49, 16, 43, 42,
-	25, 10, 40, 18, 17, 8, 35, 58, 3, 45, 52, 4, 22, 21, 60, 11,
-	53, 38, 36, 14, 46, 6, 19, 13, 28, 37, 30, 62, 61, 12, 5, 44,
-	40, 16, 35, 25, 50, 18, 34, 58, 32, 10, 49, 43, 33, 0, 56, 26,
-	9, 59, 24, 2, 1, 57, 48, 42, 54, 29, 36, 19, 6, 5, 44, 62,
-	37, 22, 20, 61, 30, 53, 3, 60, 12, 21, 14, 46, 45, 27, 52, 28,
-	24, 0, 48, 9, 34, 2, 18, 42, 16, 59, 33, 56, 17, 49, 40, 10,
-	58, 43, 8, 51, 50, 41, 32, 26, 38, 13, 20, 3, 53, 52, 28, 46,
-	21, 6, 4, 45, 14, 37, 54, 44, 27, 5, 61, 30, 29, 11, 36, 12,
-	8, 49, 32, 58, 18, 51, 2, 26, 0, 43, 17, 40, 1, 33, 24, 59,
-	42, 56, 57, 35, 34, 25, 16, 10, 22, 60, 4, 54, 37, 36, 12, 30,
-	5, 53, 19, 29, 61, 21, 38, 28, 11, 52, 45, 14, 13, 62, 20, 27,
-	57, 33, 16, 42, 2, 35, 51, 10, 49, 56, 1, 24, 50, 17, 8, 43,
-	26, 40, 41, 48, 18, 9, 0, 59, 6, 44, 19, 38, 21, 20, 27, 14,
-	52, 37, 3, 13, 45, 5, 22, 12, 62, 36, 29, 61, 60, 46, 4, 11,
-	41, 17, 0, 26, 51, 48, 35, 59, 33, 40, 50, 8, 34, 1, 57, 56,
-	10, 24, 25, 32, 2, 58, 49, 43, 53, 28, 3, 22, 5, 4, 11, 61,
-	36, 21, 54, 60, 29, 52, 6, 27, 46, 20, 13, 45, 44, 30, 19, 62,
-	25, 1, 49, 10, 35, 32, 48, 43, 17, 24, 34, 57, 18, 50, 41, 40,
-	59, 8, 9, 16, 51, 42, 33, 56, 37, 12, 54, 6, 52, 19, 62, 45,
-	20, 5, 38, 44, 13, 36, 53, 11, 30, 4, 60, 29, 28, 14, 3, 46,
-	17, 58, 41, 2, 56, 24, 40, 35, 9, 16, 26, 49, 10, 42, 33, 32,
-	51, 0, 1, 8, 43, 34, 25, 48, 29, 4, 46, 61, 44, 11, 54, 37,
-	12, 60, 30, 36, 5, 28, 45, 3, 22, 27, 52, 21, 20, 6, 62, 38
+unsigned char DES_PC1[56] = {
+	56, 48, 40, 32, 24, 16, 8,
+	0, 57, 49, 41, 33, 25, 17,
+	9, 1, 58, 50, 42, 34, 26,
+	18, 10, 2, 59, 51, 43, 35,
+	62, 54, 46, 38, 30, 22, 14,
+	6, 61, 53, 45, 37, 29, 21,
+	13, 5, 60, 52, 44, 36, 28,
+	20, 12, 4, 27, 19, 11, 3
 };
 
-static void init_SPE()
+unsigned char DES_ROT[16] = {
+	1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1
+};
+
+unsigned char DES_PC2[48] = {
+	13, 16, 10, 23, 0, 4,
+	2, 27, 14, 5, 20, 9,
+	22, 18, 11, 3, 25, 7,
+	15, 6, 26, 19, 12, 1,
+	40, 51, 30, 36, 46, 54,
+	29, 39, 50, 44, 32, 47,
+	43, 48, 38, 55, 33, 52,
+	45, 41, 49, 35, 28, 31
+};
+
+static void init_SPE(void)
 {
 	int box, index, row, column, bit;
 	ARCH_WORD mask, l, h;
@@ -366,14 +338,14 @@ static void init_SPE()
 		h = l = 0;
 		for (bit = 0; bit < 24; bit++) {
 			if (((unsigned ARCH_WORD)0x80000000 >>
-			    DES_P[DES_E[bit]]) & mask)
+			    DES_P[ARCH_INDEX(DES_E[bit])]) & mask)
 				l |= 1 << bit;
 			if (((unsigned ARCH_WORD)0x80000000 >>
-			    DES_P[DES_E[bit + 24]]) & mask)
+			    DES_P[ARCH_INDEX(DES_E[bit + 24])]) & mask)
 				h |= 1 << bit;
 		}
 
-		l = DES_TO_6_BIT(l); h = DES_TO_6_BIT(h);
+		l = DES_24_TO_32(l); h = DES_24_TO_32(h);
 
 #if ARCH_BITS >= 64
 		DES_SPE[box][index] =
@@ -386,7 +358,7 @@ static void init_SPE()
 	}
 }
 
-static void init_IP_E()
+static void init_IP_E(void)
 {
 	int src, dst, dst1, dst2;
 	int chunk, mask, value;
@@ -396,7 +368,7 @@ static void init_IP_E()
 	for (dst1 = 0; dst1 < 8; dst1++)
 	for (dst2 = 0; dst2 < 6; dst2++) {
 		dst = (dst1 << 3) + dst2;
-		src = DES_IP[DES_E[dst1 * 6 + dst2]];
+		src = DES_IP[ARCH_INDEX(DES_E[dst1 * 6 + dst2])];
 		if (src >= 32) src -= 32; else src--;
 		src ^= 7;
 
@@ -415,7 +387,7 @@ static void init_IP_E()
 	}
 }
 
-static void init_C_FP()
+static void init_C_FP(void)
 {
 	int src, dst;
 	int chunk, mask, value;
@@ -423,7 +395,7 @@ static void init_C_FP()
 	memset(DES_C_FP, 0, sizeof(DES_C_FP));
 
 	for (src = 0; src < 64; src++) {
-		dst = DES_IP[DES_C[src]] ^ 7;
+		dst = DES_IP[ARCH_INDEX(DES_C[src])] ^ 7;
 
 		chunk = src >> 2;
 		mask = 1 << (src & 3);
@@ -440,12 +412,12 @@ static void init_C_FP()
 	}
 }
 
-static void init_KS()
+static void init_KS(void)
 {
-	int pos, chr, bit, ofs, index;
+	int pos, chr, round, bit, ofs;
 	unsigned char block[64];
-	unsigned char *k;
-	ARCH_WORD value;
+	ARCH_WORD value[2];
+	int k, p, q, r, s;
 
 	memset(block, 0, sizeof(block));
 
@@ -454,22 +426,31 @@ static void init_KS()
 		for (bit = 0; bit < 7; bit++)
 			block[(pos << 3) + bit] = (chr >> (6 - bit)) & 1;
 
-		k = DES_K_bits;
-		for (index = 0; index < 32; index++) {
-			value = 0;
-			for (ofs = 0; ofs < 32; ofs += 8)
-			for (bit = 0; bit < 6; bit++) value |=
-				(ARCH_WORD)block[*k++] << (ofs + bit);
+		s = 0;
+		for (round = 0; round < 16; round++) {
+			s += DES_ROT[round];
+			value[0] = value[1] = 0;
+			k = 0;
+			for (ofs = 0; ofs < 64; ofs += 8)
+			for (bit = 0; bit < 6; bit++) {
+				p = DES_PC2[k++];
+				q = p < 28 ? 0 : 28;
+				p += s;
+				while (p >= 28) p -= 28;
+				r = DES_PC1[p + q];
+				value[ofs >> 5] |= (ARCH_WORD)block[r] <<
+					((ofs & 31) + bit);
+			}
 
 #if ARCH_BITS >= 64
-			if (index & 1)
-				DES_KS_table[pos][chr][index >> 1] |=
-					DES_DO_SIZE_FIX(value) << 32;
-			else
-				DES_KS_table[pos][chr][index >> 1] =
-					DES_DO_SIZE_FIX(value);
+			DES_KS_table[pos][chr][round] =
+				DES_DO_SIZE_FIX(value[0]) |
+				(DES_DO_SIZE_FIX(value[1]) << 32);
 #else
-			DES_KS_table[pos][chr][index] = DES_DO_SIZE_FIX(value);
+			DES_KS_table[pos][chr][round << 1] =
+				DES_DO_SIZE_FIX(value[0]);
+			DES_KS_table[pos][chr][(round << 1) + 1] =
+				DES_DO_SIZE_FIX(value[1]);
 #endif
 		}
 	}
@@ -479,7 +460,7 @@ static void init_KS()
 	memcpy(DES_KS_current, DES_KS_table, sizeof(DES_KS));
 }
 
-void DES_std_init()
+void DES_std_init(void)
 {
 	init_SPE();
 	init_IP_E();
@@ -618,12 +599,12 @@ void DES_raw_set_key(char *key)
 #endif
 
 	memcpy(DES_KS_current,
-		DES_KS_table[0][(ARCH_INDEX)(DES_key[0] = key[0] & 0x7F)],
+		DES_KS_table[0][ARCH_INDEX(DES_key[0] = key[0] & 0x7F)],
 		sizeof(DES_KS));
 
 	pos = (DES_KS *)DES_KS_table[1][0];
 	for (i = 1; i < 8; i++) {
-		DES_xor_key1(pos[(ARCH_INDEX)(DES_key[i] = key[i] & 0x7F)]);
+		DES_xor_key1(pos[ARCH_INDEX(DES_key[i] = key[i] & 0x7F)]);
 		pos += 128;
 	}
 }
@@ -635,7 +616,8 @@ void DES_std_set_key(char *key)
 	register ARCH_WORD *value1, *value2;
 #endif
 
-	for (k = i = 0; (j = key[i]) && (l = DES_key[i]); i++)
+	j = key[0];
+	for (k = i = 0; (l = DES_key[i]) && (j = key[i]); i++)
 	if (j != l) k += 3;
 
 	if (j) {
@@ -650,7 +632,7 @@ void DES_std_set_key(char *key)
 
 	if ((k < (j << 1)) && (++DES_KS_updates & 0xFFF)) {
 		j = 0; l = 1;
-		for (i = 0; (k = key[i]) && i < 8; i++) {
+		for (i = 0; i < 8 && (k = key[i]); i++) {
 			if (l)
 			if (k == (l = DES_key[j++])) continue;
 
@@ -669,7 +651,7 @@ void DES_std_set_key(char *key)
 			sizeof(DES_KS));
 
 		if (k)
-		for (i = 1; (k = key[i]) && i < 8; i++)
+		for (i = 1; i < 8 && (k = key[i]); i++)
 			DES_xor_key1(DES_KS_table[i][k & 0x7F]);
 	}
 
@@ -833,7 +815,7 @@ void DES_std_get_block(DES_binary binary, unsigned ARCH_WORD out[2])
  */
 #if DES_MASK
 #if DES_SCALE
-/* 64 bit, 4K, mask, scale */
+/* 64-bit, 4K, mask, scale */
 #define DES_ROUND(L, H, B) \
 	T = DES_INDEX(DES_SPE_F[0], DES_D & DES_MASK_6); \
 	B ^= DES_INDEX(DES_SPE_F[1], (DES_D >> 8) & DES_MASK_6); \
@@ -851,7 +833,7 @@ void DES_std_get_block(DES_binary binary, unsigned ARCH_WORD out[2])
  * (which is done at compile time), I moved the extra shift out of the mask,
  * which allows doing this with special addressing modes, where possible.
  */
-/* 64 bit, 4K, mask, no scale */
+/* 64-bit, 4K, mask, no scale */
 #define DES_ROUND(L, H, B) \
 	T = DES_INDEX(DES_SPE_F[0], (DES_D & (DES_MASK_6 >> 1)) << 1); \
 	B ^= DES_INDEX(DES_SPE_F[1], (DES_D >> 7) & DES_MASK_6); \
@@ -863,7 +845,7 @@ void DES_std_get_block(DES_binary binary, unsigned ARCH_WORD out[2])
 	B ^= T ^ DES_INDEX(DES_SPE_F[7], (DES_D >> 55) & DES_MASK_6);
 #endif
 #else
-/* 64 bit, 4K, store/load, scale */
+/* 64-bit, 4K, store/load, scale */
 #define DES_ROUND(L, H, B) \
 	T = DES_INDEX(DES_SPE_F[0], DES_B0); \
 	B ^= DES_INDEX(DES_SPE_F[1], DES_B1); \
@@ -875,7 +857,7 @@ void DES_std_get_block(DES_binary binary, unsigned ARCH_WORD out[2])
 	B ^= T ^ DES_INDEX(DES_SPE_F[7], DES_B7);
 #endif
 #else
-/* 64 bit, 128K, mask, no scale */
+/* 64-bit, 128K, mask, no scale */
 #define DES_ROUND(L, H, B) \
 	B ^= DES_INDEX(DES_SPE_W[0], DES_D & 0xFFFF); \
 	B ^= DES_INDEX(DES_SPE_W[1], (DES_D >>= 16) & 0xFFFF); \
@@ -885,7 +867,7 @@ void DES_std_get_block(DES_binary binary, unsigned ARCH_WORD out[2])
 #else
 #if !DES_128K
 #if DES_MASK
-/* 32 bit, 4K, mask */
+/* 32-bit, 4K, mask */
 #define DES_ROUND(L, H, B) \
 	L ^= DES_INDEX(DES_SPE_L[0], DES_Dl & DES_MASK_6); \
 	H ^= DES_INDEX(DES_SPE_H[0], DES_Dl & DES_MASK_6); \
@@ -904,7 +886,7 @@ void DES_std_get_block(DES_binary binary, unsigned ARCH_WORD out[2])
 	L ^= DES_INDEX(DES_SPE_L[7], (DES_Dh >>= 8) & DES_MASK_6); \
 	H ^= DES_INDEX(DES_SPE_H[7], DES_Dh & DES_MASK_6);
 #else
-/* 32 bit, 4K, store/load */
+/* 32-bit, 4K, store/load */
 #define DES_ROUND(L, H, B) \
 	L ^= DES_INDEX(DES_SPE_L[0], DES_B0); \
 	H ^= DES_INDEX(DES_SPE_H[0], DES_B0); \
@@ -924,7 +906,7 @@ void DES_std_get_block(DES_binary binary, unsigned ARCH_WORD out[2])
 	H ^= DES_INDEX(DES_SPE_H[7], DES_B7);
 #endif
 #else
-/* 32 bit, 128K, mask */
+/* 32-bit, 128K, mask */
 #define DES_ROUND(L, H, B) \
 	L ^= DES_INDEX_L(DES_SPE_W[0], DES_Dl & 0xFFFF); \
 	H ^= DES_INDEX_H(DES_SPE_W[0], DES_Dl & 0xFFFF); \
@@ -954,7 +936,7 @@ void DES_std_get_block(DES_binary binary, unsigned ARCH_WORD out[2])
 #endif
 
 #if DES_COPY
-static void crypt_body()
+static void crypt_body(void)
 #else
 void DES_std_crypt(DES_KS KS, DES_binary DES_out)
 #endif
@@ -1064,12 +1046,12 @@ static unsigned char DES_atoi64[0x100] = {
 
 ARCH_WORD DES_raw_get_salt(char *ciphertext)
 {
-	if (ciphertext[13]) return DES_atoi64[(ARCH_INDEX)ciphertext[5]] |
-		((ARCH_WORD)DES_atoi64[(ARCH_INDEX)ciphertext[6]] << 6) |
-		((ARCH_WORD)DES_atoi64[(ARCH_INDEX)ciphertext[7]] << 12) |
-		((ARCH_WORD)DES_atoi64[(ARCH_INDEX)ciphertext[8]] << 18);
-	else return DES_atoi64[(ARCH_INDEX)ciphertext[0]] |
-		((ARCH_WORD)DES_atoi64[(ARCH_INDEX)ciphertext[1]] << 6);
+	if (ciphertext[13]) return DES_atoi64[ARCH_INDEX(ciphertext[5])] |
+		((ARCH_WORD)DES_atoi64[ARCH_INDEX(ciphertext[6])] << 6) |
+		((ARCH_WORD)DES_atoi64[ARCH_INDEX(ciphertext[7])] << 12) |
+		((ARCH_WORD)DES_atoi64[ARCH_INDEX(ciphertext[8])] << 18);
+	else return DES_atoi64[ARCH_INDEX(ciphertext[0])] |
+		((ARCH_WORD)DES_atoi64[ARCH_INDEX(ciphertext[1])] << 6);
 }
 
 ARCH_WORD DES_std_get_salt(char *ciphertext)
@@ -1077,75 +1059,72 @@ ARCH_WORD DES_std_get_salt(char *ciphertext)
 	ARCH_WORD salt;
 
 	salt = DES_raw_get_salt(ciphertext);
-	salt = DES_TO_6_BIT(salt);
+	salt = DES_24_TO_32(salt);
 	return DES_DO_SIZE_FIX(salt);
 }
 
 ARCH_WORD DES_raw_get_count(char *ciphertext)
 {
-	if (ciphertext[13]) return DES_atoi64[(ARCH_INDEX)ciphertext[1]] |
-		((ARCH_WORD)DES_atoi64[(ARCH_INDEX)ciphertext[2]] << 6) |
-		((ARCH_WORD)DES_atoi64[(ARCH_INDEX)ciphertext[3]] << 12) |
-		((ARCH_WORD)DES_atoi64[(ARCH_INDEX)ciphertext[4]] << 18);
+	if (ciphertext[13]) return DES_atoi64[ARCH_INDEX(ciphertext[1])] |
+		((ARCH_WORD)DES_atoi64[ARCH_INDEX(ciphertext[2])] << 6) |
+		((ARCH_WORD)DES_atoi64[ARCH_INDEX(ciphertext[3])] << 12) |
+		((ARCH_WORD)DES_atoi64[ARCH_INDEX(ciphertext[4])] << 18);
 	else return 25;
 }
 
-ARCH_WORD *DES_raw_get_binary(char *ciphertext)
+ARCH_WORD *DES_do_IP(ARCH_WORD in[2])
 {
-	static ARCH_WORD masks[8] = {
-		0x1000, 0x0800, 0x0400, 0x0200, 0x0010, 0x0008, 0x0004, 0x0002
-	};
+	static ARCH_WORD out[2];
+	int src, dst;
 
-	static ARCH_WORD out[4];
+	out[0] = out[1] = 0;
+	for (dst = 0; dst < 64; dst++) {
+		src = DES_IP[dst ^ 0x20];
 
-	int ofs, chr, bit, index, value;
-	unsigned char block[66], *ptr;
-	ARCH_WORD mask1, mask2;
-	ARCH_WORD Rl, Rh, Ll, Lh;
-
-	if (ciphertext[13]) ofs = 9; else ofs = 2;
-	ptr = block;
-
-	for (chr = 0; chr < 11; chr++) {
-		value = DES_atoi64[(ARCH_INDEX)ciphertext[chr + ofs]];
-		mask1 = 0x20;
-
-		for (bit = 0; bit < 6; bit++) {
-			*ptr++ = value & mask1;
-			mask1 >>= 1;
-		}
+		if (in[src >> 5] & (1 << (src & 0x1F)))
+			out[dst >> 5] |= 1 << (dst & 0x1F);
 	}
 
-	Lh = Ll = Rh = Rl = 0;
-	ptr = block;
-
-	for (index = 0; index < 8; index++) {
-		mask2 = (mask1 = masks[index]) << 16;
-
-		if (*ptr++) Rl |= mask1;
-		if (*ptr++) Ll |= mask1;
-		if (*ptr++) Rl |= mask2;
-		if (*ptr++) Ll |= mask2;
-		if (*ptr++) Rh |= mask1;
-		if (*ptr++) Lh |= mask1;
-		if (*ptr++) Rh |= mask2;
-		if (*ptr++) Lh |= mask2;
-	}
-
-	out[0] = Rl;
-	out[1] = Rh;
-	out[2] = Ll;
-	out[3] = Lh;
-
 	return out;
 }
 
+ARCH_WORD *DES_raw_get_binary(char *ciphertext)
+{
+	ARCH_WORD block[3];
+	ARCH_WORD mask;
+	int ofs, chr, src, dst, value;
+
+	if (ciphertext[13]) ofs = 9; else ofs = 2;
+
+	block[0] = block[1] = 0;
+	dst = 0;
+	for (chr = 0; chr < 11; chr++) {
+		value = DES_atoi64[ARCH_INDEX(ciphertext[chr + ofs])];
+		mask = 0x20;
+
+		for (src = 0; src < 6; src++) {
+			if (value & mask)
+				block[dst >> 5] |= 1 << (dst & 0x1F);
+			mask >>= 1;
+			dst++;
+		}
+	}
+
+	return DES_do_IP(block);
+}
+
 ARCH_WORD *DES_std_get_binary(char *ciphertext)
 {
-	ARCH_WORD *out;
+	static ARCH_WORD out[4];
+	ARCH_WORD *raw;
 	ARCH_WORD salt, mask;
 
-	out = DES_raw_get_binary(ciphertext);
+	raw = DES_raw_get_binary(ciphertext);
+
+	out[0] = DES_16_TO_32(raw[0]);
+	out[1] = DES_16_TO_32(raw[0] >> 16);
+	out[2] = DES_16_TO_32(raw[1]);
+	out[3] = DES_16_TO_32(raw[1] >> 16);
 
 	out[0] = DES_DO_SIZE_FIX(out[0]);
 	out[1] = DES_DO_SIZE_FIX(out[1]);
