@@ -1,8 +1,16 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-98 by Solar Designer
+ * Copyright (c) 1996-2001 by Solar Designer
  */
 
+#ifdef __ultrix__
+#define __POSIX
+#define _POSIX_SOURCE
+#endif
+
+#ifdef _SCO_C_DIALECT
+#include <limits.h>
+#endif
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
@@ -20,7 +28,7 @@
 #include "formats.h"
 #include "bench.h"
 
-static int bench_running;
+static volatile int bench_running;
 
 static void bench_handle_timer(int signum)
 {
@@ -28,18 +36,20 @@ static void bench_handle_timer(int signum)
 }
 
 static void bench_set_keys(struct fmt_main *format,
-	struct fmt_tests **current, int cond)
+	struct fmt_tests *current, int cond)
 {
 	char *plaintext;
 	int index, length;
 
+	format->methods.clear_keys();
+
 	length = format->params.benchmark_length;
 	for (index = 0; index < format->params.max_keys_per_crypt; index++) {
 		do {
-			if (!(*current)->ciphertext)
-				*current = format->params.tests;
-			plaintext = (*current)->plaintext;
-			(*current)++;
+			if (!current->ciphertext)
+				current = format->params.tests;
+			plaintext = current->plaintext;
+			current++;
 
 			if (cond > 0) {
 				if ((int)strlen(plaintext) > length) break;
@@ -54,9 +64,13 @@ static void bench_set_keys(struct fmt_main *format,
 	}
 }
 
-int benchmark_format(struct fmt_main *format, int salts,
+char *benchmark_format(struct fmt_main *format, int salts,
 	struct bench_results *results)
 {
+	static void *binary = NULL;
+	static int binary_size = 0;
+	static char s_error[64];
+	char *where;
 	struct fmt_tests *current;
 	int cond;
 #if OS_TIMER
@@ -67,10 +81,19 @@ int benchmark_format(struct fmt_main *format, int salts,
 	unsigned ARCH_WORD count;
 	char *ciphertext;
 	void *salt, *two_salts[2];
-	int index;
+	int index, max;
 
-	if (!(current = format->params.tests)) return 1;
-	if (fmt_self_test(format)) return 1;
+	if (!(current = format->params.tests)) return "FAILED (no data)";
+	if ((where = fmt_self_test(format))) {
+		sprintf(s_error, "FAILED (%s)", where);
+		return s_error;
+	}
+
+	if (format->params.binary_size > binary_size) {
+		binary_size = format->params.binary_size;
+		binary = mem_alloc_tiny(binary_size, MEM_ALIGN_WORD);
+		memset(binary, 0x55, binary_size);
+	}
 
 	for (index = 0; index < 2; index++) {
 		two_salts[index] = mem_alloc(format->params.salt_size);
@@ -89,7 +112,7 @@ int benchmark_format(struct fmt_main *format, int salts,
 	} else
 		cond = 0;
 
-	bench_set_keys(format, &current, cond);
+	bench_set_keys(format, current, cond);
 
 #if OS_TIMER
 	memset(&it, 0, sizeof(it));
@@ -111,14 +134,18 @@ int benchmark_format(struct fmt_main *format, int salts,
 	count = 0;
 
 	index = salts;
+	max = format->params.max_keys_per_crypt;
 	do {
 		if (!--index) {
 			index = salts;
-			bench_set_keys(format, &current, cond);
+			if (!(++current)->ciphertext)
+				current = format->params.tests;
+			bench_set_keys(format, current, cond);
 		}
 
 		if (salts > 1) format->methods.set_salt(two_salts[index & 1]);
-		format->methods.crypt_all(format->params.max_keys_per_crypt);
+		format->methods.crypt_all(max);
+		format->methods.cmp_all(binary, max);
 
 		count++;
 #if !OS_TIMER
@@ -137,7 +164,7 @@ int benchmark_format(struct fmt_main *format, int salts,
 	for (index = 0; index < 2; index++)
 		mem_free(&two_salts[index]);
 
-	return event_abort ? -1 : 0;
+	return event_abort ? "" : NULL;
 }
 
 void benchmark_cps(unsigned ARCH_WORD count, clock_t time, char *buffer)
@@ -150,13 +177,12 @@ void benchmark_cps(unsigned ARCH_WORD count, clock_t time, char *buffer)
 	sprintf(buffer, cps_hi < 100 ? "%lu.%lu" : "%lu", cps_hi, cps_lo);
 }
 
-void benchmark_all()
+void benchmark_all(void)
 {
 	struct fmt_main *format;
-	char *msg_1, *msg_m;
+	char *result, *msg_1, *msg_m;
 	struct bench_results results_1, results_m;
 	char s_real[64], s_virtual[64];
-	int result;
 
 	if ((format = fmt_list))
 	do {
@@ -182,15 +208,16 @@ void benchmark_all()
 			msg_1 = "Long";
 		}
 
-		if ((result = benchmark_format(format, BENCHMARK_MANY,
+		if ((result = benchmark_format(format,
+		    format->params.salt_size ? BENCHMARK_MANY : 1,
 		    &results_m))) {
-			if (result > 0) puts("FAILED"); else putchar('\n');
+			puts(result);
 			continue;
 		}
 
 		if (msg_1)
 		if ((result = benchmark_format(format, 1, &results_1))) {
-			if (result > 0) puts("FAILED"); else putchar('\n');
+			puts(result);
 			continue;
 		}
 
@@ -198,12 +225,11 @@ void benchmark_all()
 
 		benchmark_cps(results_m.count, results_m.real, s_real);
 		benchmark_cps(results_m.count, results_m.virtual, s_virtual);
-		printf("%s:\t%s c/s"
-#if !defined(__DJGPP__) && !defined(__CYGWIN32__)
-			" real, %s c/s virtual\n",
+#if !defined(__DJGPP__) && !defined(__CYGWIN32__) && !defined(__BEOS__)
+		printf("%s:\t%s c/s real, %s c/s virtual\n",
 			msg_m, s_real, s_virtual);
 #else
-			"\n",
+		printf("%s:\t%s c/s\n",
 			msg_m, s_real);
 #endif
 
@@ -214,12 +240,11 @@ void benchmark_all()
 
 		benchmark_cps(results_1.count, results_1.real, s_real);
 		benchmark_cps(results_1.count, results_1.virtual, s_virtual);
-		printf("%s:\t%s c/s"
-#if !defined(__DJGPP__) && !defined(__CYGWIN32__)
-			" real, %s c/s virtual\n\n",
+#if !defined(__DJGPP__) && !defined(__CYGWIN32__) && !defined(__BEOS__)
+		printf("%s:\t%s c/s real, %s c/s virtual\n\n",
 			msg_1, s_real, s_virtual);
 #else
-			"\n\n",
+		printf("%s:\t%s c/s\n\n",
 			msg_1, s_real);
 #endif
 	} while ((format = format->next) && !event_abort);
