@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2001 by Solar Designer
+ * Copyright (c) 1996-2001,2003 by Solar Designer
  */
 
 #include <stdio.h>
@@ -11,6 +11,7 @@
 #include "signals.h"
 #include "compiler.h"
 #include "loader.h"
+#include "logger.h"
 #include "status.h"
 #include "recovery.h"
 #include "config.h"
@@ -144,7 +145,10 @@ void do_external_crack(struct db_main *db)
 	unsigned char *internal;
 	c_int *external;
 
+	log_event("Proceeding with external mode: %.100s", ext_mode);
+
 	if (!f_generate) {
+		log_event("! No generate() function defined");
 		fprintf(stderr, "No generate() for external mode: %s\n",
 			ext_mode);
 		error();
