@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2002 by Solar Designer
+ * Copyright (c) 1996-2003 by Solar Designer
  */
 
 #include <stdio.h>
@@ -71,6 +71,47 @@ static void john_register_all(void)
 	}
 }
 
+static void john_log_format(void)
+{
+	int min_chunk, chunk;
+
+	log_event("- Hash type: %.100s (lengths up to %d%s)",
+		database.format->params.format_name,
+		database.format->params.plaintext_length,
+		database.format->methods.split != fmt_default_split ?
+		", longer passwords split" : "");
+
+	log_event("- Algorithm: %.100s",
+		database.format->params.algorithm_name);
+
+	chunk = min_chunk = database.format->params.max_keys_per_crypt;
+	if (options.flags & (FLG_SINGLE_CHK | FLG_BATCH_CHK) &&
+	    chunk < SINGLE_HASH_MIN)
+			chunk = SINGLE_HASH_MIN;
+	if (chunk > 1)
+		log_event("- Candidate passwords %s be buffered and "
+			"tried in chunks of %d",
+			min_chunk > 1 ? "will" : "may",
+			chunk);
+}
+
+static char *john_loaded_counts(void)
+{
+	static char s_loaded_counts[80];
+
+	if (database.password_count == 1)
+		return "1 password hash";
+
+	sprintf(s_loaded_counts,
+		database.salt_count > 1 ?
+		"%d password hashes with %d different salts" :
+		"%d password hashes with no different salts",
+		database.password_count,
+		database.salt_count);
+
+	return s_loaded_counts;
+}
+
 static void john_load(void)
 {
 	struct list_entry *current;
@@ -80,12 +121,12 @@ static void john_load(void)
 	if (options.flags & FLG_EXTERNAL_CHK)
 		ext_init(options.external);
 
-	if (options.flags & FLG_MAKECHARS_CHK) {
+	if (options.flags & FLG_MAKECHR_CHK) {
 		options.loader.flags |= DB_CRACKED;
 		ldr_init_database(&database, &options.loader);
 
 		if (options.flags & FLG_PASSWD) {
-			ldr_show_pot_file(&database, LOG_NAME);
+			ldr_show_pot_file(&database, POT_NAME);
 
 			database.options->flags |= DB_PLAINTEXTS;
 			if ((current = options.passwd->head))
@@ -94,7 +135,7 @@ static void john_load(void)
 			} while ((current = current->next));
 		} else {
 			database.options->flags |= DB_PLAINTEXTS;
-			ldr_show_pot_file(&database, LOG_NAME);
+			ldr_show_pot_file(&database, POT_NAME);
 		}
 
 		return;
@@ -113,7 +154,7 @@ static void john_load(void)
 			options.loader.flags |= DB_CRACKED;
 			ldr_init_database(&database, &options.loader);
 
-			ldr_show_pot_file(&database, LOG_NAME);
+			ldr_show_pot_file(&database, POT_NAME);
 
 			if ((current = options.passwd->head))
 			do {
@@ -142,26 +183,30 @@ static void john_load(void)
 			ldr_load_pw_file(&database, current->data);
 		} while ((current = current->next));
 
-		ldr_load_pot_file(&database, LOG_NAME);
+		if ((options.flags & FLG_CRACKING_CHK) &&
+		    database.password_count) {
+			log_init(LOG_NAME, NULL, options.session);
+			if (status_restored_time)
+				log_event("Continuing an interrupted session");
+			else
+				log_event("Starting a new session");
+			log_event("Loaded a total of %s", john_loaded_counts());
+		}
+
+		ldr_load_pot_file(&database, POT_NAME);
 
 		ldr_fix_database(&database);
 
-		printf("Loaded %d password%s%s",
-			database.password_count,
-			database.password_count != 1 ? "s" : "",
-			database.password_count ? "" : ", exiting...");
-		if (database.password_count > 1) {
-			printf(" with ");
-			printf(database.salt_count != 1 ? "%d" : "no",
-				database.salt_count);
-			printf(" different salts");
-		}
-		if (database.password_count)
-			printf(" (%s [%s])\n",
+		if (database.password_count) {
+			log_event("Remaining %s", john_loaded_counts());
+			printf("Loaded %s (%s [%s])\n",
+				john_loaded_counts(),
 				database.format->params.format_name,
 				database.format->params.algorithm_name);
-		else
-			putchar('\n');
+		} else {
+			log_discard();
+			puts("No password hashes loaded");
+		}
 
 		if ((options.flags & FLG_PWD_REQ) && !database.salts) exit(0);
 	}
@@ -210,11 +255,17 @@ static void john_run(void)
 	if (options.flags & FLG_TEST_CHK)
 		benchmark_all();
 	else
-	if (options.flags & FLG_MAKECHARS_CHK)
+	if (options.flags & FLG_MAKECHR_CHK)
 		do_makechars(&database, options.charset);
 	else
 	if (options.flags & FLG_CRACKING_CHK) {
-		if (!(options.flags & FLG_STDOUT)) log_init(LOG_NAME);
+		if (!(options.flags & FLG_STDOUT)) {
+			log_init(LOG_NAME, POT_NAME, options.session);
+			john_log_format();
+			if (cfg_get_bool(SECTION_OPTIONS, NULL, "Idle"))
+				log_event("- Configured to use otherwise idle "
+					"processor cycles only");
+		}
 		tty_init();
 
 		if (options.flags & FLG_SINGLE_CHK)
@@ -235,7 +286,6 @@ static void john_run(void)
 
 		status_print();
 		tty_done();
-		if (!(options.flags & FLG_STDOUT)) log_done();
 	}
 }
 
@@ -243,7 +293,15 @@ static void john_done(void)
 {
 	path_done();
 
-	check_abort();
+	if ((options.flags & FLG_CRACKING_CHK) &&
+	    !(options.flags & FLG_STDOUT)) {
+		if (event_abort)
+			log_event("Session aborted");
+		else
+			log_event("Session completed");
+	}
+	log_done();
+	check_abort(0);
 }
 
 int main(int argc, char **argv)
