@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-99 by Solar Designer
+ * Copyright (c) 1996-99,2003 by Solar Designer
  */
 
 #include <stdio.h>
@@ -14,6 +14,7 @@
 #include "path.h"
 #include "signals.h"
 #include "loader.h"
+#include "logger.h"
 #include "status.h"
 #include "recovery.h"
 #include "rpp.h"
@@ -51,7 +52,9 @@ static void restore_line_number(void)
 
 	for (line_number = 0; line_number < rec_pos; line_number++)
 	if (!fgets(line, sizeof(line), word_file)) {
-		if (ferror(word_file)) pexit("fgets"); else {
+		if (ferror(word_file))
+			pexit("fgets");
+		else {
 			fprintf(stderr, "fgets: Unexpected EOF\n");
 			error();
 		}
@@ -81,7 +84,9 @@ static void fix_state(void)
 	else
 	if ((rec_pos = ftell(word_file)) < 0) {
 #ifdef __DJGPP__
-		if (rec_pos != -1) rec_pos = 0; else
+		if (rec_pos != -1)
+			rec_pos = 0;
+		else
 #endif
 			pexit("ftell");
 	}
@@ -101,7 +106,9 @@ static int get_progress(void)
 
 	if ((pos = ftell(word_file)) < 0) {
 #ifdef __DJGPP__
-		if (pos != -1) pos = 0; else
+		if (pos != -1)
+			pos = 0;
+		else
 #endif
 			pexit("ftell");
 	}
@@ -123,10 +130,12 @@ void do_wordlist_crack(struct db_main *db, char *name, int rules)
 {
 	char line[LINE_BUFFER_SIZE];
 	struct rpp_context ctx;
-	char *rule, *word;
+	char *prerule, *rule, *word;
 	char last[RULE_WORD_SIZE];
 	char *(*apply)(char *word, char *rule, int split);
 
+	log_event("Proceeding with wordlist mode");
+
 	if (name) {
 		if (!(word_file = fopen(path_expand(name), "r")))
 			pexit("fopen: %s", path_expand(name));
@@ -135,8 +144,11 @@ void do_wordlist_crack(struct db_main *db, char *name, int rules)
 
 	length = db->format->params.plaintext_length;
 
+	log_event("- Wordlist file: %.100s", path_expand(name));
+
 	if (rules) {
 		if (rpp_init(rule_ctx = &ctx, SUBSECTION_WORDLIST)) {
+			log_event("! No wordlist mode rules found");
 			fprintf(stderr, "No wordlist mode rules found in %s\n",
 				cfg_name);
 			error();
@@ -145,11 +157,15 @@ void do_wordlist_crack(struct db_main *db, char *name, int rules)
 		rules_init(length);
 		rule_count = rules_count(&ctx, -1);
 
+		log_event("- %d preprocessed word mangling rules", rule_count);
+
 		apply = rules_apply;
 	} else {
 		rule_ctx = NULL;
 		rule_count = 1;
 
+		log_event("- No word mangling rules");
+
 		apply = dummy_rules_apply;
 	}
 
@@ -162,14 +178,30 @@ void do_wordlist_crack(struct db_main *db, char *name, int rules)
 
 	crk_init(db, fix_state, NULL);
 
-	if (rules) rule = rpp_next(&ctx); else rule = "";
+	if (rules) prerule = rpp_next(&ctx); else prerule = "";
+	rule = "";
 
 	memset(last, ' ', length + 1);
 	last[length + 2] = 0;
 
-	if (rule)
+	if (prerule)
 	do {
-		if (!rules || (rule = rules_reject(rule, db)))
+		if (rules) {
+			if ((rule = rules_reject(prerule, db))) {
+				if (strcmp(prerule, rule))
+					log_event("- Rule #%d: '%.100s'"
+						" accepted as '%.100s'",
+						rule_number + 1, prerule, rule);
+				else
+					log_event("- Rule #%d: '%.100s'"
+						" accepted",
+						rule_number + 1, prerule);
+			} else
+				log_event("- Rule #%d: '%.100s' rejected",
+					rule_number + 1, prerule);
+		}
+
+		if (rule)
 		while (fgetl(line, sizeof(line), word_file)) {
 			line_number++;
 
